/**
 * 
 */
package org.jvnet.hifaces.application.navigation.impl;

import java.util.HashMap;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jvnet.hifaces.counter.Counter;

public class State {

	private static Log logger = LogFactory.getLog(State.class);

	private final Long id;

	private final String viewId;

	private Transition nextTransition;

	private Transition previousTransition;

	private boolean entered = false;

	private State(final Long id, final String viewId) {
		this.id = id;
		this.viewId = viewId;
	}

	public Long getId() {
		return id;
	}

	public String getViewId() {
		return viewId;
	}

	public State getPrevious() {
		return previousTransition == null ? null : previousTransition
				.getTo();
	}

	public Transition getPreviousTransition() {
		return previousTransition;
	}

	public Transition getNextTransition() {
		return nextTransition;
	}

	public boolean isPreviousAvailable() {
		return previousTransition != null;
	}

	public State getNext() {
		return nextTransition == null ? null : nextTransition.getTo();
	}

	public boolean isNextAvailable() {
		return nextTransition != null;
	}

	public void redirectNext(State to) {
		nextTransition = new Transition(this, to, true);
		to.previousTransition = new Transition(to, this, true);
	}

	public void forwardNext(State to) {
		nextTransition = new Transition(this, to, false);
		to.previousTransition = new Transition(to, this, false);
	}

	public void redirectPrevious(State from) {
		previousTransition = new Transition(from, this, true);
		from.nextTransition = new Transition(this, from, true);
	}

	public void forwardPrevious(State from) {
		previousTransition = new Transition(from, this, false);
		from.nextTransition = new Transition(this, from, false);
	}

	@Override
	public String toString() {
		return "<" + getId() + "|" + getViewId() + ">";
	}

	public static final String STATE_COUNTER_SESSION_ATTRIBUTE_NAME = "_hiStateCounter";

	public static final String STATE_MAP_SESSION_ATTRIBUTE_NAME = "_hiStateMap";

	private static Counter getStateCounter(FacesContext facesContext) {
		return Counter.getCounter(facesContext,
				STATE_COUNTER_SESSION_ATTRIBUTE_NAME);
	}

	public static State copyPrevious(FacesContext facesContext, State state) {
		final State copy = createState(facesContext, state.getViewId());
		if (state.isPreviousAvailable()) {
			final State previousCopy = copyPrevious(facesContext, state
					.getPrevious());
			if (state.getPreviousTransition().isRedirect()) {
				previousCopy.redirectNext(copy);
			} else {
				previousCopy.forwardNext(copy);
			}
		}
		return copy;
	}

	public static State copyNext(FacesContext facesContext, State state) {
		final State copy = createState(facesContext, state.getViewId());
		if (state.isNextAvailable()) {
			final State nextCopy = copyNext(facesContext, state.getNext());
			if (state.getNextTransition().isRedirect()) {
				nextCopy.redirectPrevious(copy);
			} else {
				nextCopy.forwardPrevious(copy);
			}
		}
		return copy;
	}

	public static State copy(FacesContext facesContext, State state) {
		final State copy = createState(facesContext, state.getViewId());
		if (state.isPreviousAvailable()) {
			final State previousCopy = copyPrevious(facesContext, state
					.getPrevious());
			if (state.getPreviousTransition().isRedirect()) {
				previousCopy.redirectNext(copy);
			} else {
				previousCopy.forwardNext(copy);
			}
		}
		if (state.isNextAvailable()) {
			final State nextCopy = copyNext(facesContext, state.getNext());
			if (state.getNextTransition().isRedirect()) {
				nextCopy.redirectPrevious(copy);
			} else {
				nextCopy.forwardPrevious(copy);
			}
		}
		return copy;
	}

	private static Long nextStateId(FacesContext facesContext) {
		return getStateCounter(facesContext).nextId();
	}

	public static State createState(FacesContext facesContext, String viewId) {
		final State state = new State(nextStateId(facesContext), viewId);
		getStateMap(facesContext).put(state.getId(), state);
		return state;
	}

	public static State getState(FacesContext facesContext, Long stateId) {
		return getStateMap(facesContext).get(stateId);
	}

	private static Map<Long, State> getStateMap(FacesContext facesContext) {
		final Map sessionMap = facesContext.getExternalContext()
				.getSessionMap();
		final Object draftStateMap = sessionMap
				.get(STATE_MAP_SESSION_ATTRIBUTE_NAME);

		if (draftStateMap != null) {
			return (Map<Long, State>) draftStateMap;
		} else {
			final Map<Long, State> stateMap = new HashMap<Long, State>();
			sessionMap.put(STATE_MAP_SESSION_ATTRIBUTE_NAME, stateMap);
			return stateMap;
		}
	}
	
	public static class Transition {

		public final State from;

		public final State to;

		public final boolean redirect;

		private Transition(State from, State to, boolean redirect) {
			this.from = from;
			this.to = to;
			this.redirect = redirect;
		}

		public State getFrom() {
			return from;
		}

		public State getTo() {
			return to;
		}

		public boolean isRedirect() {
			return redirect;
		}
	}
	
}