package org.jvnet.hifaces.event;

import java.lang.ref.Reference;

import javax.swing.event.ChangeEvent;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;

public class TreeModelListenerList extends AbstractWeakListenerList {

	public void addChangeListener(TreeModelListener listener) {
		add(listener);
	}

	public void removeChangeListener(TreeModelListener listener) {
		remove(listener);
	}

	public void clearChangeListeners() {
		clear();
	}

	public void fireTreeNodesChanged(TreeModelEvent event) {
		synchronized (list) {
			final Reference[] references = (Reference[]) list
					.toArray(new Reference[list.size()]);
			for (int index = 0; index < references.length; index++) {
				final Reference reference = references[index];
				final Object referent = reference.get();
				if (referent == null) {
					list.remove(referent);
				} else {
					final TreeModelListener listener = (TreeModelListener) referent;
					listener.treeNodesChanged(event);
				}
			}
		}
	}

	public void fireTreeNodesInserted(TreeModelEvent event) {
		synchronized (list) {
			final Reference[] references = (Reference[]) list
					.toArray(new Reference[list.size()]);
			for (int index = 0; index < references.length; index++) {
				final Reference reference = references[index];
				final Object referent = reference.get();
				if (referent == null) {
					list.remove(referent);
				} else {
					final TreeModelListener listener = (TreeModelListener) referent;
					listener.treeNodesInserted(event);
				}
			}
		}
	}

	public void fireTreeNodesRemoved(TreeModelEvent event) {
		synchronized (list) {
			final Reference[] references = (Reference[]) list
					.toArray(new Reference[list.size()]);
			for (int index = 0; index < references.length; index++) {
				final Reference reference = references[index];
				final Object referent = reference.get();
				if (referent == null) {
					list.remove(referent);
				} else {
					final TreeModelListener listener = (TreeModelListener) referent;
					listener.treeNodesRemoved(event);
				}
			}
		}
	}

	public void fireTreeStructureChanged(TreeModelEvent event) {
		synchronized (list) {
			final Reference[] references = (Reference[]) list
					.toArray(new Reference[list.size()]);
			for (int index = 0; index < references.length; index++) {
				final Reference reference = references[index];
				final Object referent = reference.get();
				if (referent == null) {
					list.remove(referent);
				} else {
					final TreeModelListener listener = (TreeModelListener) referent;
					listener.treeStructureChanged(event);
				}
			}
		}
	}

	protected void fire(Object listenerDraft, Object eventDraft) {
	}
}
