package org.jvnet.hifaces.lifecycle;

import javax.faces.FacesException;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseListener;
import javax.faces.lifecycle.Lifecycle;
import javax.servlet.http.HttpSession;

import org.jvnet.hifaces.io.util.TransientValue;

public class SessionSynchronizedLifecycle extends Lifecycle {

	public static final String MUTEXT_ATTRIBUTE_NAME = "_hiSSLExecuteMutex";

	private Lifecycle parent;

	public SessionSynchronizedLifecycle(Lifecycle parent) {
		this.parent = parent;
	}

	private Object getOrCreateMutex(FacesContext facesContext) {
		return getOrCreateMutex(facesContext, MUTEXT_ATTRIBUTE_NAME);
	}

	private Object getOrCreateMutex(FacesContext facesContext, String name) {
		final Object draftSession = facesContext.getExternalContext()
				.getSession(true);
		final HttpSession session = (HttpSession) draftSession;

		final Object mutex;
		synchronized (session) {
			final TransientValue<Object> value = (TransientValue<Object>) session
					.getAttribute(name);
			if (value != null && value.get() != null) {
				mutex = value.get();
			} else {
				mutex = new Object();
				final TransientValue<Object> newValue = new TransientValue<Object>(
						mutex);
				session.setAttribute(name, newValue);
			}
		}
		return mutex;
	}

	@Override
	public void execute(FacesContext facesContext) throws FacesException {
		final Object mutex = getOrCreateMutex(facesContext);
		synchronized (mutex) {
			parent.execute(facesContext);
		}
	}

	@Override
	public void render(FacesContext facesContext) throws FacesException {
		final Object mutex = getOrCreateMutex(facesContext);
		synchronized (mutex) {
			parent.render(facesContext);
		}
	}

	@Override
	public void addPhaseListener(PhaseListener listener) {
		parent.addPhaseListener(listener);

	}

	@Override
	public PhaseListener[] getPhaseListeners() {
		return parent.getPhaseListeners();
	}

	@Override
	public void removePhaseListener(PhaseListener listener) {
		parent.removePhaseListener(listener);
	}

}
