package org.jvnet.hifaces.renderkit.html;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Iterator;

import javax.faces.application.ViewHandler;
import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.component.UIParameter;
import javax.faces.component.html.HtmlCommandLink;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.apache.myfaces.shared.renderkit.RendererUtils;
import org.apache.myfaces.shared.renderkit.html.HTML;
import org.apache.myfaces.shared.renderkit.html.HtmlRenderer;
import org.apache.myfaces.shared.renderkit.html.HtmlRendererUtils;
import org.jvnet.hifaces.component.html.HtmlOpenWindowLink;

public class HtmlOpenWindowLinkRenderer extends HtmlRenderer {
	public static final String URL_STATE_MARKER = "JSF_URL_STATE_MARKER=DUMMY";

	public static final int URL_STATE_MARKER_LEN = URL_STATE_MARKER.length();

	// private static final Log log = LogFactory.getLog(HtmlLinkRenderer.class);

	public boolean getRendersChildren() {
		// We must be able to render the children without a surrounding anchor
		// if the Link is disabled
		return true;
	}

	public void decode(FacesContext facesContext, UIComponent component) {
		super.decode(facesContext, component); // check for NP

		if (component instanceof HtmlOpenWindowLink) {
			// do nothing
		} else {
			throw new IllegalArgumentException("Unsupported component class "
					+ component.getClass().getName());
		}
	}

	public void encodeBegin(FacesContext facesContext, UIComponent component)
			throws IOException {
		super.encodeBegin(facesContext, component); // check for NP

		if (component instanceof HtmlOpenWindowLink) {
			renderOutputLinkStart(facesContext, (HtmlOpenWindowLink) component);
		} else {
			throw new IllegalArgumentException("Unsupported component class "
					+ component.getClass().getName());
		}
	}

	public void encodeChildren(FacesContext facesContext, UIComponent component)
			throws IOException {
		RendererUtils.renderChildren(facesContext, component);
	}

	public void encodeEnd(FacesContext facesContext, UIComponent component)
			throws IOException {
		super.encodeEnd(facesContext, component); // check for NP

		if (component instanceof UIOutput) {
			renderOutputLinkEnd(facesContext, component);
		} else {
			throw new IllegalArgumentException("Unsupported component class "
					+ component.getClass().getName());
		}
	}

	private void addChildParametersToHref(UIComponent linkComponent,
			StringBuffer hrefBuf, boolean firstParameter, String charEncoding)
			throws IOException {
		for (Iterator it = getChildren(linkComponent).iterator(); it.hasNext();) {
			UIComponent child = (UIComponent) it.next();
			if (child instanceof UIParameter) {
				String name = ((UIParameter) child).getName();
				Object value = ((UIParameter) child).getValue();

				addParameterToHref(name, value, hrefBuf, firstParameter,
						charEncoding);
				firstParameter = false;
			}
		}
	}

	protected void renderOutputLinkStart(FacesContext facesContext,
			HtmlOpenWindowLink output) throws IOException {
		ResponseWriter writer = facesContext.getResponseWriter();

		// calculate href
		String href = org.apache.myfaces.shared.renderkit.RendererUtils
				.getStringValue(facesContext, output);
		if (getChildCount(output) > 0) {
			StringBuffer hrefBuf = new StringBuffer(href);
			addChildParametersToHref(output, hrefBuf,
					(href.indexOf('?') == -1), // first url parameter?
					writer.getCharacterEncoding());
			href = hrefBuf.toString();
		}
		href = facesContext.getApplication().getViewHandler().getResourceURL(facesContext, href); // TODO:
		// or
		// encodeActionURL
		// ?

		writer.startElement(HTML.SCRIPT_ELEM, output);
		writer.writeAttribute(HTML.TYPE_ATTR, "text/javascript", null);

		final String cliendId = output.getClientId(facesContext);

		final String jsClientId = cliendId.replace(':', '$');

		final String windowName = output.getWindowName();
		final String windowFeatures = output.getWindowFeatures();
		final String windowReplace = output.getWindowReplace();

		final String functionName = "Hi$openWindow$" + jsClientId;

		writer.write("function " + functionName + "()\n");
		writer.write("{\n");
		writer.write("var win = window.open(");
		
		if (href != null)
		{
			writer.write("'" + href + "'");
			
			if (windowName != null)
			{
				writer.write(", '" + windowName + "'");
				
				if (windowFeatures != null)
				{
					writer.write(", '" + windowFeatures + "'");
					if (windowReplace != null)
					{
						writer.write(", " + windowReplace );
					}
				}
			}
		}

		writer.write(");\n");
		writer.write("win.opener = self;\n");
		writer.write("win.focus();\n");
		writer.write("}");

		// writer.write("Hi$installAdjustRows(\'"+
		// uiComponent.getClientId(facesContext)+"\');");
		// writer.write("alert(\'hi there!\');");
		writer.endElement(HTML.SCRIPT_ELEM);

		String clientId = output.getClientId(facesContext);

		// write anchor
		writer.startElement(HTML.ANCHOR_ELEM, output);
		writer.writeAttribute(HTML.ID_ATTR, clientId, null);
		writer.writeAttribute(HTML.NAME_ATTR, clientId, null);
		writer.writeURIAttribute(HTML.HREF_ATTR, "javascript:" + functionName + "();", null);
		HtmlRendererUtils
				.renderHTMLAttributes(
						writer,
						output,
						org.apache.myfaces.shared.renderkit.html.HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES);
		writer.flush();
	}

	private static void addParameterToHref(String name, Object value,
			StringBuffer hrefBuf, boolean firstParameter, String charEncoding)
			throws UnsupportedEncodingException {
		if (name == null) {
			throw new IllegalArgumentException(
					"Unnamed parameter value not allowed within link.");
		}

		hrefBuf.append(firstParameter ? '?' : '&');
		hrefBuf.append(URLEncoder.encode(name, charEncoding));
		hrefBuf.append('=');
		if (value != null) {
			// UIParameter is no ConvertibleValueHolder, so no conversion
			// possible
			hrefBuf.append(URLEncoder.encode(value.toString(), charEncoding));
		}
	}

	protected void renderOutputLinkEnd(FacesContext facesContext,
			UIComponent component) throws IOException {
		ResponseWriter writer = facesContext.getResponseWriter();
		// force separate end tag
		writer.writeText("", null);
		writer.endElement(HTML.ANCHOR_ELEM);
	}
}
