package org.jvnet.hifaces.util;

import java.io.IOException;
import java.io.Serializable;
import java.io.ByteArrayInputStream;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class SerializableDomElement implements Serializable {

  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  /**
   * Document builder.
   */
  private static transient DocumentBuilder DOCUMENT_BUILDER;

  /**
   * Transformer.
   */
  private static transient Transformer TRANSFORMER;

  /**
   * Wrapped document.
   */
  private transient Element element;

  /**
   * Document string - used to save document for serialization.
   */
  private String documentString;

  // Static initialization
  static {
    try {
      // Create document builder factory
      final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
      // Turn namespace awareness on
      documentBuilderFactory.setNamespaceAware(true);
      // Create new document builder
      DOCUMENT_BUILDER = documentBuilderFactory.newDocumentBuilder();
      TRANSFORMER = TransformerFactory.newInstance().newTransformer();
    }
    catch (ParserConfigurationException pcex) {
      // Should never happen
      throw new ExceptionInInitializerError(pcex);
    }
    catch (TransformerConfigurationException tex) {
      // Should never happen
      throw new ExceptionInInitializerError(tex);
    }
  }

  public SerializableDomElement(final Element element) {
    this.element = element;
  }

  public Element getElement() {
    return element;
  }

  private void readObject(java.io.ObjectInputStream in)
      throws IOException,
      ClassNotFoundException,
      SAXException {
    in.defaultReadObject();
    final Document document;
    synchronized (DOCUMENT_BUILDER) {
      document = DOCUMENT_BUILDER.parse(new InputSource(new StringReader(documentString)));
    }
    element = document.getDocumentElement();
    documentString = null;
  }

  private void writeObject(java.io.ObjectOutputStream out) throws IOException {
    final StringWriter sw = new StringWriter();
    synchronized (TRANSFORMER) {
      try {
        TRANSFORMER.transform(new DOMSource(element), new StreamResult(sw));
      }
      catch (TransformerException tex) {
        throw new IOException(tex.getMessageAndLocation());
      }
    }
    documentString = sw.toString();
    element = null;
    out.defaultWriteObject();
  }
}